@@ -19,7 +19,7 @@
 /* VTE accessibility object.  Based heavily on inspection of libzvt's
  * accessibility code. */
 
-#ident "$Id: vteaccess.c,v 1.20 2002/10/30 20:36:58 nalin Exp $"
+#ident "$Id: vteaccess.c,v 1.21 2002/12/18 20:02:42 nalin Exp $"
 #include "../config.h"
 #include <atk/atk.h>
 #include <gtk/gtk.h>
@@ -33,9 +33,7 @@
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
 #else
-#define _(String) String
 #define bindtextdomain(package,dir)
 #endif
 
